<?php


namespace Drupal\alien_alias;

use Drupal\alien_alias\Exogen\ExogenInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Site\Settings;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class AlienAliasAccesses
 *
 * @package Drupal\alien_alias
 */
final class AlienAliasAccesses {

  /**
   * Flags permitting, we store this information in the database.
   *
   * @param \Drupal\alien_alias\Exogen\ExogenInterface $exogen
   *   This alien alias.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current page request.
   * @param int|null $time
   */
  public static function Record(ExogenInterface $exogen, Request $request, ?int $time = NULL): void {
    // Are we stat-keeping at all? Is it switched on for this alien alias?
    if (Settings::get('alien_alias_keep_stats', TRUE) && $exogen->isKeepStats()) {
      // Make sure the stat name exists as a key in the request query.
      if ($request->query->has($exogen->getStatName())) {
        // Insert the data into the table.
        $sql = "INSERT INTO {alien_alias_stats} (`aaid`, `value`, `ip_addr`, `created`) VALUES (:aaid, :value, :ip_addr, :created)";
        Database::getConnection()->query($sql, [
          ':aaid' => $exogen->getId(),
          ':value' => $request->query->get($exogen->getStatName()),
          ':ip_addr' => $request->getClientIp() ?? '',
          ':created' => $time ?: time(),
        ]);
      }
    }
  }

  /**
   * Flags permitting, return an array of the query parameters to go.
   *
   * @param ExogenInterface $exogen
   *   This alien alias.
   * @param Request $request
   *   The current page request.
   *
   * @return mixed[]
   *   An array of the query parameters to go.
   */
  public static function QueryParams(ExogenInterface $exogen, Request $request): array {
    $params = [];

    if ($exogen->isPassthrough()) {
      // Get all the query parameters.
      $params = $request->query->getIterator()->getArrayCopy();

      // Remove the stat query parameter (if any).
      if ($statName = $exogen->getStatName()) {
        unset($params[$statName]);
      }
    }

    return $params;
  }


}
